@@ -1,10 +1,15 @@
 package Mojo::DOM::Role::Analyzer ;
-$Mojo::DOM::Role::Analyzer::VERSION = '0.010';
+$Mojo::DOM::Role::Analyzer::VERSION = '0.011';
 use strict;
 use warnings;
 use Role::Tiny;
 use Carp;
-use Log::Log4perl::Shortcuts qw(:all);
+
+around find => sub {
+  my $orig = shift;
+  my $self = shift;
+  return $self->$orig(@_)->with_roles('+Extra');
+};
 
 use overload "cmp" => sub { $_[0]->compare(@_) }, fallback => 1;
 
@@ -13,28 +18,6 @@ sub element_count {
   return $self->descendant_nodes->grep(sub { $_->type eq 'tag' })->size;
 }
 
-sub parent_all {
-  my $self    = shift;
-  my $tag     = shift;
-  carp 'No tag passed to parent_all method' unless $tag;
-
-  my $t_count = $self->find($tag)->size;
-
-  my $enclosing_tag = $self->at($tag);
-  my $current_t_count = 0;
-  while ($current_t_count < $t_count) {
-    my $parent = $enclosing_tag->parent;
-    $current_t_count = $parent->find('p')->size;
-    $enclosing_tag = $parent;
-  }
-  return $enclosing_tag;
-}
-
-sub parent_ptags {
-  my $self = shift;
-  return $self->parent_all('p');
-}
-
 sub _get_selectors {
   my ($s, $sel1, $sel2);
   if (!$_[2]) {
@@ -53,6 +36,15 @@ sub _get_selectors {
   return ($s, $sel1, $sel2);
 }
 
+sub is_ancestor_to {
+  my $s = shift;
+  my $arg = shift;
+  my $sel1 = $s->selector;
+  my $sel2 = $arg->selector;
+
+  return $sel2 =~ /^\Q$sel1\E/ ? 1 : 0;
+}
+
 # traverses the DOM upward to find the closest tag node
 sub closest_up {
   return _closest(@_, 'up');
@@ -91,9 +83,9 @@ sub _closest {
 
   my @sorted = sort { $s->root->at($a) cmp $s->root->at($b) } @selectors;
   if ($dir eq 'up') {
-    return $s->root->at($sorted[-1]);  # get furthers from the top (closest to node of interest)
+    return $s->root->at($sorted[-1]);  # get furtherest from the top (closest to node of interest)
   } else {
-    return $s->root->at($sorted[0]);   # get futherst from the bottom (closest to node of interest)
+    return $s->root->at($sorted[0]);   # get futherest from the bottom (closest to node of interest)
   }
 
 }
@@ -119,7 +111,7 @@ sub compare {
 sub distance {
   my ($s, $sel1, $sel2) = _get_selectors(@_);
 
-  my $common = common($s, $s->root->at($sel1), $s->root->at($sel2));
+  my $common = $s->common($s->root->at($sel1), $s->root->at($sel2));
   my $dist_leg1 = $s->root->at($sel1)->depth - $common->depth;
   my $dist_leg2 = $s->root->at($sel2)->depth - $common->depth;
 
@@ -143,71 +135,65 @@ sub deepest {
   return $deepest_depth;
 }
 
-# find the common ancestor between two nodes
+sub common_parent_within {
+  my $s = shift;
+  my $sel = $s->selector;
+  $s = $s->root->at($_[0]);
+  $s->common($sel);
+}
+
+# find the common ancestor between a node and another node or group of nodes
 sub common {
+# uncomment to debug
+# use Log::Log4perl::Shortcuts qw(:all); # for development only
+#  if (ref $_[0]) { logd ref $_[0]; } else { logd $_[0]; }
+#  if (ref $_[1]) { logd ref $_[1]; } else { logd $_[1]; }
+#  if (ref $_[2]) { logd ref $_[2]; } else { logd $_[2]; }
+
+   # The argument handling is a bit confusing. Keep these important notes in mind while reading this code:
+
+   # 1) This method is called on Mojo::DOM objects (obviously)
+   # 2) Don't confuse this mthod with its sister method also named "common"
+   #    in Mojo::DOM::Collection::Extra which works with Mojo::Collection objects
+   # 3) The argument handling below works for the different types of common syntaxes noted
+   #    below in the comments.
+
   my ($s, $sel1, $sel2);
-  if ($_[1] && $_[2] && !ref $_[1] && !ref $_[2]) {
-    ($s, $sel1, $sel2) = @_;
+
+  # function-like use of common: $dom->commont($dom1, $dom2)
+  if (ref $_[1] && ref $_[2]) {
+    $s = $_[0];
+    $sel1 = $_[1]->selector;
+    $sel2 = $_[2]->selector;
+  # DWIM syntax handling
   } else {
-    ($s, $sel1, $sel2) = _get_selectors(@_);
+    if (!$_[1] && !$_[2]) {                         # $dom->at('div');
+      my $s = shift;
+      return $s->root->find($s->selector)->common;
+    } elsif ($_[1] && !ref $_[1] && !$_[2]) {       # $dom->at('div.first')->common('p');
+      $s = shift;
+      $sel1 = $s->selector;
+      $sel2 = $s->root->at(shift)->selector;
+    }
   }
 
   my @t1_path = split / > /, $sel1;
   my @t2_path = split / > /, $sel2;
 
-  my @last_common;
-  foreach my $p1 (@t1_path) {
-    my $p2 = shift(@t2_path);
-    if ($p1 eq $p2) {
-      push @last_common, $p1;
-    } else {
-      last;
-    }
+  my @common_path;
+  foreach my $seg (@t1_path) {
+    my $seg2 = shift @t2_path;
+    last if !$seg2 || $seg ne $seg2;
+    push @common_path, $seg2;
   }
-  my $common_selector = join ' > ', @last_common;
+
+  my $common_selector = join ' > ', @common_path;
 
   return $s->root->at($common_selector);
 
 }
 
-#sub _dump_dom {
-  #use Log::Log4perl::Shortcuts qw(:all);
-#  my $dom = shift;
-#  logd $dom->tag;
-#  logd $dom->attr();
-  #$}
-
-#sub _gsec {
-#  my $enclosing_tag = shift;
-#  my @sub_enclosing_nodes;
-#  my $node_counter = 0;
-#  my $largest = 0;
-#  my $largest_node = 0;
-#  foreach my $c ($enclosing_tag->children('*')->each) {
-#    my $size = $c->find('p')->size;
-#    if ($size > $largest) {
-#      $largest = $size;
-#      $largest_node = $node_counter;
-#    }
-#    my $same_depth = 1;
-#    my $depth_tracker = undef;
-#    my $depth_total;
-#    foreach my $p ($c->find('p')->each) {
-#      my $sel = $p->selector;
-#      my @parts = split /\s>\s/, $sel;
-#      my $parts = scalar @parts;
-#      if ($depth_tracker && $parts != $depth_tracker) {
-#        $same_depth = 0;
-#      }
-#      $depth_tracker = $parts;
-#      $depth_total += $parts;
-#    }
-#    push @sub_enclosing_nodes, { child => $c->selector, size => $size, avg_ptag_depth => ($depth_total / $size), all_ptags_have_same_depth => $same_depth };
-#  }
-#  return @sub_enclosing_nodes;
-#}
-#
-
+# get secondary enclosing tags
 sub _gsec {
   my $s            = shift;
   my $selector     = shift;
@@ -254,14 +240,20 @@ sub tag_analysis {
   carp "A selector argument must be passed to the tag_analysis method"
     unless $selector;
 
-  my @sub_enclosing_nodes = $s->_tag_analysis_helper('p');
+  my @sub_enclosing_nodes = $s->_tag_analysis_helper($selector);
 
   foreach my $sn (@sub_enclosing_nodes) {
     next if $sn->{all_tags_have_same_depth};
-    my $ec = $s->at($sn->{selector})->parent_all('p');
-    my @enclosing_nodes = $ec->_tag_analysis_helper('p');
+    my $ec = $s->at($sn->{selector})->common($selector);
+    my @enclosing_nodes = $ec->_tag_analysis_helper($selector);
     push @sub_enclosing_nodes, @enclosing_nodes;
   }
+
+  # cleanup any unnecessary nodes at top of the array wrapping the smallest enconpassing dom
+  my $total_tags = $s->find($selector)->size;
+  my $number_of_tags = grep { $_->{size} == $total_tags } @sub_enclosing_nodes;
+  splice @sub_enclosing_nodes, 0, $number_of_tags - 1;
+
   return @sub_enclosing_nodes;
 }
 
@@ -301,7 +293,7 @@ Mojo::DOM::Role::Analyzer - miscellaneous methods for analyzing a DOM
   my $count = $analyzer->at('body')->element_count;
 
   # get the smallest containing dom object that contains all the paragraph tags
-  my $containing_dom = $analyzer->parent_ptags;
+  my $containing_dom = $analyzer->common_ancestor('p');
 
   # compare DOM objects to see which comes first in the document
   my $tag1 = $analyzer->at('p.first');
@@ -334,7 +326,7 @@ the dom. See C<compare> method below for return values.
 
 =head2 Methods
 
-=head4 closest_down
+=head3 closest_down
 
   my $closest_down_dom = $dom->at('h1')->closest_down('p');
 
@@ -346,10 +338,10 @@ below the tag node of interest (or, in the case of L<closeest_up> lowest in the
 DOM but still above the tag node of interest), not by the shortest distance
 (number of "hops") to the other node.
 
-For example, in the code below, the C<E<lt>h1<E<gt>> tag containing "Heading 1"
-is five hops away from the C<E<lt>p<E<gt>> tag, while the other
-C<E<lt>h1<E<gt>> tag is only two hops away. But despite being more hops away,
-the C<E<lt>h1<E<gt>> tag containing "Header 1" is considered to be closer.
+For example, in the code below, the C<E<lt>h1E<gt>> tag containing "Heading 1"
+is five hops away from the C<E<lt>pE<gt>> tag, while the other
+C<E<lt>h1E<gt>> tag is only two hops away. But despite being more hops away,
+the C<E<lt>h1E<gt>> tag containing "Header 1" is considered to be closer.
 
     <p>Paragraph</p>
     <div><div><div><div><h1>Heading 1</h1></div></div></div></div>
@@ -373,19 +365,33 @@ is calculated.
 
 =head4 C<$dom-E<gt>common($selector_str1, $selector_str2)>
 
-  my $common_dom = $dom->at('div.bar')->common('div.foo');    # 'div.foo' is relative to root
+=head4 C<$dom-E<gt>at($tag1)-E<gt>common>
+
+  # Find the common ancestor for two nodes
+  my $common $dom->at('div.bar')->common('div.foo');    # 'div.foo' is relative to root
 
   # OR
 
+  # Pass in two $dom objects
   my $dom1 = $dom->at('div.bar');
   my $dom2 = $dom->at('div.foo');
   my $common = $dom->common($dom1, $dom2);
 
   # OR
 
+  # Pass in two selectors
   my $common = $dom->common($dom->at('p')->selector, $dom->at('h1')->selector);
 
-Returns the lowest common ancestor node between two tag nodes or two selector strings.
+  # OR
+
+  # Find the common ancestor for all paragraph nodes with class "foo"
+  # This syntax is a wrapper for the Mojo::Collection::Role::Extra->common method
+  my $common = $dom->at('p.foo')->common;
+
+Returns the lowest common ancestor node between two nodes or
+betwween a node and a group of nodes sharing the same selector.
+
+See L<Mojo::Collection::Role::Extra/common> for a similar method that is on Mojo::Collections.
 
 =head3 compare
 
@@ -453,36 +459,18 @@ determine the total distance between the two nodes.
 Returns the number of elements in a dom object, including children of children
 of children, etc.
 
-=head3 parent_all
+=head3 is_ancestor_to
 
-  $dom = $dom->parent_all('a');                     # finds parent within root that contains all 'a' tags
-  $dom = $dom->at('div.article')->parent_all('ul'); # finds parent within C<div.article> that has all 'ul' tags
+  $is_ancestor = $s->('h1')->is_ancestor_to('p.foo');
 
-Returns the smallest containing $dom within the $dom the method is called on
-that wraps all the tags nodes matching the selector given by the argument.
-
-Example:
-
-  $dom = Mojo::DOM->with_roles('+Analyzer')->new(
-    '<div class="one"><div class="two"><p>foo</p><p>bar</p></div></div>');
-
-  my $parent = $dom->parent->all('p');
-  # $parent now has a $dom with C<E<lt>div class="two"E<gt>> as its root
-
-=head3 parent_ptags
-
-  $dom = $dom->parent_ptags;
-  $dom = $dom->at('div.article')->parent_ptags;
-
-A conveniece method that works like the C<parent_all> method but automatically supplies a
-C<'p'> tag argument for you.
+Returns true if a node is an ancestor to another node, false otherwise.
 
 =head3 tag_analysis
 
   @enclosing_tags = $dom->tag_analysis('p');
 
 Searches through a DOM for tag nodes that enclose tags matching the given
-selector (see L<parent_all> method) and returns an array of hash references
+selector (see L<common_ancestor> method) and returns an array of hash references
 with the following information for each of the enclosing nodes:
 
   {
@@ -494,7 +482,7 @@ with the following information for each of the enclosing nodes:
 
 =head1 VERSION
 
-version 0.010
+version 0.011
 
 =for :stopwords cpan testmatrix url annocpan anno bugtracker rt cpants kwalitee diff irc mailto metadata placeholders metacpan
 
@@ -533,6 +521,11 @@ L<https://github.com/sdondley/Mojo-DOM-Role-Analyzer>
 
   git clone git://github.com/sdondley/Mojo-DOM-Role-Analyzer.git
 
+=head1 SEE ALSO
+
+L<Mojo::DOM>
+L<Mojo::Collection::Role::Extra>
+
 =head1 AUTHOR
 
 Steve Dondley <s@dondley.com>
